using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class GameState : MonoBehaviour {
	public Text messageText;
	public static bool DEBUG_MODE = true;
	public const int MAX_PLAYERS = 5;

	public bool FORCE_SINGLE_PLAYER = false;
	public int foodChunksToWin = 50;
	public AudioSource winAudio;

	[HideInInspector] public bool startedPlaying = false;

	int numPlayers = 1;

	// Static singleton property
	public static GameState Instance { get; private set; }

	void Awake()
	{
		// First we check if there are any other instances conflicting
		if(Instance != null && Instance != this)
		{
			//destroy the new instance
			Destroy( this );
			return;
		}
		// Here we save our singleton instance
		Instance = this;
		
		// Furthermore we make sure that we don't destroy between scenes (this is optional)
//		DontDestroyOnLoad(gameObject);

		Init();
	}

	public void Init() {
		LeanTween.init( 200 );
		numPlayers = Input.GetJoystickNames().Length + 1;
	}

	void Start() {
	}

	public void StartPlaying () {
		PowerupSpawner[] spawners = FindObjectsOfType<PowerupSpawner>();
		foreach ( PowerupSpawner s in spawners ) {
			s.StartSpawning();
		}
	}


	void Update() {
		RunCommands();
#if UNITY_EDITOR
		if ( DEBUG_MODE ) {
			RunDebugCommands();
		}
#endif
	}

	public void Win( PlayerWorm playerWorm ) {
		GameGUI.Instance.DeclareWinner(playerWorm);
		//LeanTween.scale(playerWorm.gameObject, Vector3.one * 5f, 2f);
		playerWorm.moveSpeed = 7f;
		playerWorm.AddSegments(50);
		playerWorm.SetDefaultWiggleAmount(5f);
		playerWorm.wiggleFrequency = 20f;
		winAudio.Play();
		Invoke("RestartLevel", 10f);
	}


	public void RestartLevel() {
		SceneManager.LoadScene( SceneManager.GetActiveScene().buildIndex );
	}

	void ExitToMenu() {
	}

	void RunCommands() {
		if ( Input.GetKeyDown( "escape" ) || Input.GetKeyDown( "f10" ) ) {
			Application.Quit();
			ExitToMenu();
		}
		if ( Input.GetKeyDown( "r" ) ) {
			RestartLevel();
		}

		if ( Input.GetKeyDown( "f12" ) ) {
			TakeScreenshot( 2 );
		}
	}

	void RunDebugCommands() {
		if (Input.GetKeyDown("f1")) {
			FORCE_SINGLE_PLAYER = !FORCE_SINGLE_PLAYER;
		}

		if ( Input.GetKeyDown( "t" ) ) {
			if ( Time.timeScale == 1.0f ) {
				Time.timeScale = 10.0f;
			}
			else {
				Time.timeScale = 1.0f;
			}
		}
		if ( Input.GetKeyDown( "y" ) ) {
			if ( Time.timeScale == 1.0f ) {
				Time.timeScale = 0.25f;
			}
			else {
				Time.timeScale = 1.0f;
			}
		}
		//if ( Input.GetKeyDown( "f" ) ) {
		//	if ( Time.timeScale != 0f ) {
		//		Time.timeScale = 0f;
		//	}
		//	else {
		//		Time.timeScale = 1.0f;
		//	}
		//}
	}

	public void TakeScreenshot( int supersize = 1 ) {
		StartCoroutine( TakeScreenshot_Internal( supersize ) );
	}

	IEnumerator TakeScreenshot_Internal( int supersize = 1 ) {
		yield return new WaitForEndOfFrame();
		string path = "Screenshots";
		bool newDirectory = !System.IO.Directory.Exists( path );
		if ( newDirectory ) System.IO.Directory.CreateDirectory( path );

		int fileCount = System.IO.Directory.GetFiles( path, "*.png" ).Length;
		string filename;
		int i = 0;
		do {
			filename = path + "/Screenshot_" + ( fileCount + i ).ToString( "00" ) + ".png";
			i++;
		} while ( System.IO.File.Exists( filename ) );
		ScreenCapture.CaptureScreenshot( filename, supersize );
		Debug.Log( string.Format( "Took screenshot to: {0}", filename ) );
	}






	public void ShowMessage( string _text, float time = 0.0f ) {
		messageText.enabled = true;
		messageText.text = _text;
		if ( time > 0 ) {
			StartCoroutine( HideMessage( time ) );
		}
	}

	public void HideMessage() {
		messageText.enabled = false;
	}
	
	IEnumerator HideMessage( float time ) {
		yield return new WaitForSeconds( time );
		messageText.enabled = false;
	}
}
