﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using Rewired;
using System.ComponentModel;

public enum PlayerNum { P1=0,P2,P3,P4,P5}

public class PlayerWorm : LittleMissBehaviour {
	#region CONSTS
	public const float SEGMENT_SPACING = .1f;
	const float BITE_INTERVAL = .4f;
	#endregion

	#region STATIC
	public static float TRAIL_Z_DEPTH = 0f;
	#endregion

	#region INSPECTOR
	public PlayerNum playerNum;
	public Transform headPrefab;	
	public Sprite segmentSprite, bandSprite;
	public Color headColor, segmentColor, bandColor, trailColor;
	public WormSegment wormSegmentPrefab;
	public int numStartingWormSegments = 5;
	public TrailRenderer dirtTrailPrefab;
	[Range(.01f,10f)]
	public float moveSpeed = .5f;
	[Range(.001f, 5f)]
	public float wiggleAmount = .01f;
	[Range(1f, 20f)]
	public float wiggleFrequency = 5f;
	public AudioClip wormBiteClip, wormOuchClip, speedBoostClip, fillGaugeClip, addSegmentsClip, subtractSegmentsClip;
	public AudioSource extraAudio;

	[Header("SPEED")]
	public float speedBoostAmount = 3f;
	public float speedBoostDuration = 2f;
	public float speedBoostCooldown = 10f;


	[Header("DEBUG - DON'T EDIT")]
	public int numSegments = 0;
	public float moveSpeedTrailMult = 1f;
	public float moveSpeedTrailMultResetTimer = -1f;
	public float speedBoostMult = 1f;
	public float speedBoostCooldownTimer = -1f;
	public float speedBoostDurationTimer = -1f;
	public float lengthSpeedMult = 1f;
	#endregion

	#region VARS
	private Rewired.Player rewiredPlayer { get { return ReInput.isReady ? ReInput.players.GetPlayer((int)playerNum) : null; } }
	SpriteRenderer eyes, pupils, blink;
	WormSegment headSegment, tailSegment;
	List<WormSegment> wormSegments;
	[HideInInspector] public bool isMoving = false;
	TrailRenderer trail;
	float lastBiteSoundTime = -1f;
	float defaultWiggleAmount;
	bool isEating = false;
	Veggie activeVeggieOnTopOf = null;
	float lastEatTime = -1f, lastVegSetTime = -1f;
	bool canUseSpeedBoost = true;
	float lastOuchSoundTime = -1f;
	#endregion

	private void Awake() {
		//rewiredPlayer = ReInput.players.GetPlayer((int)playerNum);
	}

	void Start () {
		defaultWiggleAmount = wiggleAmount;
		wormSegments = new List<WormSegment>(numStartingWormSegments);
		wormSegmentPrefab.CreatePool(numStartingWormSegments * 2);
		//start near head and go to tail
		for ( int i = 0; i < numStartingWormSegments; i++ ) {
			var segment = wormSegmentPrefab.Spawn(transform.position - transform.up * i * SEGMENT_SPACING);
			segment.worm = this;
			segment.transform.parent = transform;
			wormSegments.Add(segment);
			if (i == 0) {
				headSegment = segment;
				headSegment.name = "head";
				//headSegment.SetColor(headColor);
				segment.SetSprite(null);
				var head = headPrefab.Spawn(segment.transform.position);
				head.parent = segment.transform;
				head.localPosition = Vector3.zero;
				pupils = head.Find("pupils").GetComponent<SpriteRenderer>();
				eyes = head.Find("eyes").GetComponent<SpriteRenderer>();
				blink = head.Find("blink").GetComponent<SpriteRenderer>();
			}
			else {
				//gradient
				//segment.SetColor( Color.Lerp( segmentColor, trailColor, (float)i/(float)numStartingWormSegments));
				//segment.SetColor(segmentColor);
				segment.SetSprite(segmentSprite);
			}

			bool isBand = (i > 3 && i < 7);
			if ( isBand ) {
				//segment.SetSprite(bandSprite);
				segment.SetColor(bandColor);
				segment.transform.localScale = Vector3.one * 1.1f;
			}

			segment.nextWormSegment = i == 0 ? null : wormSegments[i - 1];
			if ( i == numStartingWormSegments - 1) {
				tailSegment = wormSegments[i];
				trail = dirtTrailPrefab.Spawn();
				trail.startColor = trail.endColor = trailColor;
				trail.transform.SetParent(wormSegments[i].transform, false);
			}
		}

		StartCoroutine(Blink());
	}

	public bool IsHead( WormSegment segment ) {
		return segment == headSegment;
	}

	public bool IsTail(WormSegment segment) {
		return segment == tailSegment;
	}

	public void AddSegments( int num ) {
		for ( int i = 0; i < num; i++ ) {
			AddSegment();
		}
		extraAudio.PlayOneShot(addSegmentsClip, .35f);
	}

	public void AddSegment() {
		var segment = wormSegmentPrefab.Spawn(tailSegment.transform.position /*- tailSegment.transform.up * SEGMENT_SPACING*/); //commented out part makes tail pop in
		segment.worm = this;
		segment.transform.parent = transform;
		segment.SetSprite(segmentSprite);
		//segment.SetColor(segmentColor);
		wormSegments.Add(segment);
		tailSegment = segment;
		segment.nextWormSegment = wormSegments[wormSegments.Count - 2];
		trail.transform.SetParent(segment.transform, false);
	}

	public void RemoveSegments(int num) {
		for (int i = 0; i < num; i++) {
			RemoveSegment();
		}
		PlayOuchClip();
	}

	public void RemoveSegment() {
		if ( wormSegments.Count <= 8 ) {
			return;
        }
		WormSegment segmentToBeRemoved = wormSegments[wormSegments.Count - 1];
		segmentToBeRemoved.nextWormSegment = null;
		wormSegments.RemoveAt(wormSegments.Count - 1);
		tailSegment = wormSegments[wormSegments.Count - 1];
		trail.transform.SetParent(tailSegment.transform, false);
		StartCoroutine(RemoveSegment_Internal(segmentToBeRemoved));
	}

	IEnumerator RemoveSegment_Internal(WormSegment segmentToBeRemoved) {
		//lerp to new tail
		float distToNewTail = 1f;
		while (distToNewTail > .01f) {
			segmentToBeRemoved.transform.position = Vector2.MoveTowards(segmentToBeRemoved.transform.position, tailSegment.transform.position, GetCurrMoveSpeed() * 2.5f * Time.deltaTime);
			segmentToBeRemoved.transform.SetZ(PlayerWorm.TRAIL_Z_DEPTH);
			distToNewTail = Vector2.Distance(segmentToBeRemoved.transform.position, tailSegment.transform.position);
			yield return null;
        }
		segmentToBeRemoved.Recycle();
	}

	public void SpeedBoost () {
		if ( !canUseSpeedBoost ) {
			return;
		}
		speedBoostMult = speedBoostAmount;
		speedBoostCooldownTimer = speedBoostCooldown;
		speedBoostDurationTimer = speedBoostDuration;
		canUseSpeedBoost = false;
		for ( int i = 0; i < wormSegments.Count; i++ ) {
			wormSegments[i].FlashLong();
		}
		extraAudio.PlayOneShot(speedBoostClip, .5f);
	}

	void SpeedBoostReady() {
		canUseSpeedBoost = true;
		for (int i = 0; i < wormSegments.Count; i++) {
			wormSegments[i].FlashQuick();
		}
	}

	float wiggleVel = 0f, speedBoostVel = 0f;
	void Update () {
		if ( !GameState.Instance.startedPlaying ) {
			return;
		}
		if (!ReInput.isReady) return; // Exit if Rewired isn't ready. This would only happen during a script recompile in the editor.
		if (rewiredPlayer == null) return;

		numSegments = wormSegments.Count;

		//VEG/EAT
		if (Time.time - lastVegSetTime > .4f) {
			activeVeggieOnTopOf = null;
		}
		if ( Time.time - lastEatTime > .1f ) {
			isEating = false;
		}

		//MOVE
		Vector2 move = Vector2.zero;

		//NEW REWIRED INPUT
		move.x = rewiredPlayer.GetAxis("Move Horizontal");
		move.y = rewiredPlayer.GetAxis("Move Vertical");

		if (rewiredPlayer.GetButtonDown( "Action1")) {
			SpeedBoost();
		}
#if UNITY_EDITOR
		if ( playerNum == PlayerNum.P1 && Input.GetKey( KeyCode.F ) ) {
			AddSegment();
		}
#endif

		move.Normalize();

		isMoving = move.x != 0f || move.y != 0f;

		//MOVE SPEED MULT
		if ( moveSpeedTrailMultResetTimer > 0f ) {
			moveSpeedTrailMultResetTimer -= Time.deltaTime;
			if ( moveSpeedTrailMultResetTimer <= 0f ) {
				moveSpeedTrailMult = 1f;
            }
		}
		//SPEED BOOST
		if (speedBoostCooldownTimer > 0f ) {
			speedBoostCooldownTimer -= Time.deltaTime;
		}
		if (speedBoostCooldownTimer < 0f && !canUseSpeedBoost) {
			SpeedBoostReady();
		}
		if (speedBoostDurationTimer > 0f ) {
			speedBoostDurationTimer -= Time.deltaTime;
		}
		if (speedBoostDurationTimer <= 0f) {
			speedBoostMult = Mathf.SmoothDamp(speedBoostMult, 1f, ref speedBoostVel, .25f);
		}

		//AUDIO
		if ( isMoving && !audio.isPlaying ) {
			audio.Play();
		}
		else if ( !isMoving && audio.isPlaying ) {
			audio.Pause();
		}

		//WIGGLE
		wiggleAmount = Mathf.SmoothDamp(wiggleAmount, defaultWiggleAmount, ref wiggleVel, .1f);
		Vector2 wiggle = isMoving ? (Vector2)(wiggleAmount * headSegment.transform.right * Mathf.Sin(wiggleFrequency * Time.time)) : Vector2.zero;
		move += wiggle;

		Vector2 projectedPos = headSegment.transform.position + (Vector3)move * GetCurrMoveSpeed() * Time.deltaTime;

		//Grapher.Log(wiggle.magnitude, "wiggle", Color.blue);
		//Util.DrawLine(headSegment.transform.position, (Vector2)headSegment.transform.position + (projectedPos - (Vector2)headSegment.transform.position).normalized * .5f, Color.red);

		//COLLISION
		if (!Dirt.Instance.IsInside(projectedPos)) { //outside of level bounds
			//don't move
		}
		else if (Dirt.Instance.IsInObstacle(projectedPos) != null) {			
			//don't move
		}
		else {
			headSegment.transform.position = projectedPos;

			bool isCrossingAnotherPlayerTrail = false;
			int numSegmentsCrossingSelfTrail = 0;
			for ( int i = 0; i < wormSegments.Count; i++ ) {
				bool isHead = wormSegments[i] == headSegment;
				bool isTail = wormSegments[i] == tailSegment;
				if ( (isHead || isTail) && !isEating && activeVeggieOnTopOf != null ) { //not ideal - means if head is on a veggie then tail will stop marking ownership even if tail is way off of the veg. but gotta do it this way for now
					//don't eat if we can't but are on top of veg!
					continue;
				}
				int prevOwnerOrEaten = Dirt.Instance.UpdateWormSegmentPos(playerNum, wormSegments[i].transform.position, isHead, isTail, isEating);
				if (isHead && prevOwnerOrEaten == Dirt.TILE_EATEN_ID && isEating) {
					//means we just ate this tile
					FillWithFoodChunk();
				}
				else if ( prevOwnerOrEaten != (int)playerNum && prevOwnerOrEaten >= 0 ) {
					isCrossingAnotherPlayerTrail = true;
				}
				else if ( prevOwnerOrEaten == (int)playerNum && i < wormSegments.Count - 3) { //discard last couple segments here otherwise it thinks its always crossing self
					numSegmentsCrossingSelfTrail++;
				}
			}

			//if (isCrossingAnotherPlayerTrail) { //this means we just ran over another worm's trail
			//	moveSpeedTrailMult = .75f;
			//	moveSpeedTrailMultResetTimer = 3f;
			//}
			if (numSegmentsCrossingSelfTrail > 0 ) {
				moveSpeedTrailMult = Mathf.Lerp(1.25f, 2f, (float)numSegmentsCrossingSelfTrail / wormSegments.Count);
				moveSpeedTrailMultResetTimer = .5f;
			}
		}

		if ( isMoving ) {
			headSegment.transform.rotation = Quaternion.FromToRotation(Vector3.up, move);
			//TRAIL_Z_DEPTH -= Time.deltaTime * .001f;
			//transform.SetZ(TRAIL_Z_DEPTH);
		}

		//TRAIL
		//if ( !Dirt.Instance.IsInside( trail.transform.position ) ) {
		//}

		//PUPILS
		if (isMoving) {
			pupils.transform.SetLocalPosX(.007f * Mathf.Sin(3f * GetCurrMoveSpeed() * Time.time));
		}

		//BLINK
	}

	public float GetCurrMoveSpeed() {
		float eatingMult = isEating ? .5f : 1f;
		//faster if shorter, slower if longer - currently 8-10 = faster, 10-50 slows down linearly
		lengthSpeedMult = wormSegments.Count < numStartingWormSegments ? 1.25f : Mathf.Lerp(1f, .85f, (float)(wormSegments.Count - numStartingWormSegments) / (numStartingWormSegments * 4));
		return moveSpeed * eatingMult * moveSpeedTrailMult * speedBoostMult * lengthSpeedMult;
    }

	#region EATING
	public void SetVeggie( Veggie veg ) {
		//this is what we're on top of, but doesn't mean we are eating it necessarily if full
		activeVeggieOnTopOf = veg;
		lastVegSetTime = Time.time;
	}

	public bool CanEat() {
		return isMoving && IsEmptyFoodSlots();
	}

	public void Eat() {
		wiggleAmount = 1f;
		isEating = true;
		lastEatTime = Time.time;
		if ( CanPlayBiteSound() ) {
			lastBiteSoundTime = Time.time;
			audio.PlayOneShot(wormBiteClip, 1f);
		}
	}

	public bool CanPlayBiteSound() {
		return Time.time - lastBiteSoundTime >= BITE_INTERVAL;
	}

	void FillWithFoodChunk() {
		for ( int i = 1; i < wormSegments.Count; i++ ) {
			if ( !wormSegments[i].IsFilledWithFood()) {
				wormSegments[i].FillWithFood(activeVeggieOnTopOf);
				return;
			}
		}
	}

	bool IsEmptyFoodSlots() {
		for (int i = 1; i < wormSegments.Count; i++) {
			if (!wormSegments[i].IsFilledWithFood()) {
				return true;
			}
		}
		return false;
	}

	public int EmptyOutFoodChunks() {
		int foodChunksRemoved = 0;
		for (int i = 1; i < wormSegments.Count; i++) {
			if (wormSegments[i].IsFilledWithFood()) {
				wormSegments[i].RemoveFood();
				foodChunksRemoved++;
			}
		}
		return foodChunksRemoved;
	}
	#endregion

	public void PlayOuchClip() {
		if ( Time.time - lastOuchSoundTime > 1f ) {
			lastOuchSoundTime = Time.time;
			if ( Util.Cointoss() ) {
				extraAudio.PlayOneShot(subtractSegmentsClip, .5f);
			}
			else {
				extraAudio.PlayOneShot(wormOuchClip, .25f);
			}
		}
	}

	public void PlayFillGaugeClip() {
		extraAudio.PlayOneShot(fillGaugeClip, .5f);
	}

	IEnumerator Blink() {
		while ( true ) {
			yield return new WaitForSeconds(Random.Range( 2f, 7f ));
			eyes.enabled = pupils.enabled = false;
			blink.enabled = true;
			yield return new WaitForSeconds(.2f);
			eyes.enabled = pupils.enabled = true;
			blink.enabled = false;
		}
	}

	public void SetDefaultWiggleAmount( float val ) {
		defaultWiggleAmount = val;
	}
	
}
