﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PowerupSpawner : LittleMissBehaviour
{
    public Powerup powerupPrefab;
    public Transform botLeft, topRight;
    public float minTime, maxTime;
    public int maxSpawnedAtOnce = 10;

    Rect spawnRect;
    List<Powerup> spawnedPowerups;

    void Start() {
        spawnRect = new Rect();
        spawnRect.xMin = botLeft.position.x;
        spawnRect.xMax = topRight.position.x;
        spawnRect.yMin = botLeft.position.y;
        spawnRect.yMax = topRight.position.y;

        powerupPrefab.CreatePool(30);

        spawnedPowerups = new List<Powerup>(100);

        //StartSpawning();
    }

    public void Stop() {
        StopAllCoroutines();
    }

    public void StartSpawning() {
        StartCoroutine(StartSpawning_Internal());
    }

    IEnumerator StartSpawning_Internal() {
        while ( true ) {
            yield return new WaitForSeconds(Random.Range(minTime, maxTime));
            if (GetCount() < maxSpawnedAtOnce) {
                Vector2 pos = new Vector2(Random.Range(spawnRect.xMin, spawnRect.xMax), Random.Range(spawnRect.yMin, spawnRect.yMax));
                var powerup = powerupPrefab.Spawn(pos);
                spawnedPowerups.Add(powerup);
            }
        }
    }

    int GetCount() {
        int cnt = 0;
        for ( int i = 0; i < spawnedPowerups.Count; i++ ) {
            if ( spawnedPowerups[i].gameObject.activeInHierarchy) {
                cnt++;
            }
        }
        return cnt;
    }
}
