﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Diagnostics;
#if UNITY_EDITOR
using UnityEditor;
#endif

public class WormSegment : LittleMissBehaviour
{
    public PlayerWorm worm;
    public WormSegment nextWormSegment;
    public SpriteRenderer flashSprite;
    public float flashAlpha;
    public SpriteRenderer foodChunk;

    private void OnEnable() {
        foodChunk.enabled = false;
    }

    void Update() {
        if (nextWormSegment == null)
            return;
        transform.position = Vector2.MoveTowards(transform.position, nextWormSegment.transform.position - nextWormSegment.transform.up * PlayerWorm.SEGMENT_SPACING, worm.GetCurrMoveSpeed() * 2f * Time.deltaTime);
        transform.SetZ(PlayerWorm.TRAIL_Z_DEPTH);

        if (worm.isMoving) {
            Vector3 vectorToTarget = nextWormSegment.transform.position - transform.position;
            float angle = Mathf.Atan2(vectorToTarget.y, vectorToTarget.x) * Mathf.Rad2Deg - 90f;
            Quaternion q = Quaternion.AngleAxis(angle, Vector3.forward);
            transform.rotation = Quaternion.Slerp(transform.rotation, q, Time.deltaTime * worm.GetCurrMoveSpeed() * 20f );
        }
    }

    public void SetSprite( Sprite sprite ) {
        spriteRenderer.sprite = sprite;
    }

    public void SetColor( Color color ) {
        spriteRenderer.color = color;
    }

    public void FlashQuick() {
        StartCoroutine(FlashQuick_Internal());
    }

    IEnumerator FlashQuick_Internal() {
        LeanTween.alpha(flashSprite.gameObject, flashAlpha, .1f).setEase(LeanTweenType.easeInSine);
        yield return new WaitForSeconds(.1f);
        LeanTween.alpha(flashSprite.gameObject, 0f, .1f).setEase(LeanTweenType.easeOutSine);
    }

    public void FlashLong() {        
        StartCoroutine(FlashLong_Internal());
    }

    IEnumerator FlashLong_Internal() {
        LeanTween.alpha(flashSprite.gameObject, flashAlpha, .1f).setEase(LeanTweenType.easeInSine);
        yield return new WaitForSeconds(.1f);
        LeanTween.alpha(flashSprite.gameObject, 0f, worm.speedBoostDuration - .1f ).setEase(LeanTweenType.easeOutSine);
    }

    public void FillWithFood( Veggie veg ) {
        foodChunk.color = veg.foodColor;
        foodChunk.enabled = true;
    }

    public bool IsFilledWithFood() {
        return foodChunk.enabled;
    }

    public void RemoveFood() {
        foodChunk.enabled = false;
    }

#if UNITY_EDITOR
    private void OnDrawGizmos() {
        Util.DrawLine(transform.position, transform.position + transform.up * .2f, Color.green);
    }
#endif
}
